@@ -1306,7 +1306,4 @@ CheckPointLogicalRewriteHeap(void)
 		}
 	}
 	FreeDir(mappings_dir);
-
-	/* persist directory entries to disk */
-	fsync_fname("pg_logical/mappings", true);
 }
