@@ -4,7 +4,7 @@
  *	  implementation of insert algorithm
  *
  *
- * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -17,6 +17,7 @@
 
 #include "access/genam.h"
 #include "access/spgist_private.h"
+#include "access/spgxlog.h"
 #include "access/xloginsert.h"
 #include "miscadmin.h"
 #include "storage/bufmgr.h"
@@ -188,7 +189,7 @@ saveNodeLink(Relation index, SPPageDesc *parent,
 	SpGistInnerTuple innerTuple;
 
 	innerTuple = (SpGistInnerTuple) PageGetItem(parent->page,
-								PageGetItemId(parent->page, parent->offnum));
+												PageGetItemId(parent->page, parent->offnum));
 
 	spgUpdateNodeLink(innerTuple, parent->node, blkno, offnum);
 
@@ -200,7 +201,7 @@ saveNodeLink(Relation index, SPPageDesc *parent,
  */
 static void
 addLeafTuple(Relation index, SpGistState *state, SpGistLeafTuple leafTuple,
-		   SPPageDesc *current, SPPageDesc *parent, bool isNulls, bool isNew)
+			 SPPageDesc *current, SPPageDesc *parent, bool isNulls, bool isNew)
 {
 	spgxlogAddLeaf xlrec;
 
@@ -221,7 +222,7 @@ addLeafTuple(Relation index, SpGistState *state, SpGistLeafTuple leafTuple,
 		/* Tuple is not part of a chain */
 		leafTuple->nextOffset = InvalidOffsetNumber;
 		current->offnum = SpGistPageAddNewItem(state, current->page,
-										   (Item) leafTuple, leafTuple->size,
+											   (Item) leafTuple, leafTuple->size,
 											   NULL, false);
 
 		xlrec.offnumLeaf = current->offnum;
@@ -249,7 +250,7 @@ addLeafTuple(Relation index, SpGistState *state, SpGistLeafTuple leafTuple,
 		OffsetNumber offnum;
 
 		head = (SpGistLeafTuple) PageGetItem(current->page,
-							  PageGetItemId(current->page, current->offnum));
+											 PageGetItemId(current->page, current->offnum));
 		if (head->tupstate == SPGIST_LIVE)
 		{
 			leafTuple->nextOffset = head->nextOffset;
@@ -262,7 +263,7 @@ addLeafTuple(Relation index, SpGistState *state, SpGistLeafTuple leafTuple,
 			 * and set new second element
 			 */
 			head = (SpGistLeafTuple) PageGetItem(current->page,
-							  PageGetItemId(current->page, current->offnum));
+												 PageGetItemId(current->page, current->offnum));
 			head->nextOffset = offnum;
 
 			xlrec.offnumLeaf = offnum;
@@ -466,7 +467,7 @@ moveLeafs(Relation index, SpGistState *state,
 		for (i = 0; i < nDelete; i++)
 		{
 			it = (SpGistLeafTuple) PageGetItem(current->page,
-								  PageGetItemId(current->page, toDelete[i]));
+											   PageGetItemId(current->page, toDelete[i]));
 			Assert(it->tupstate == SPGIST_LIVE);
 
 			/*
@@ -504,7 +505,7 @@ moveLeafs(Relation index, SpGistState *state,
 	 * be any concurrent scan so we need not provide a redirect.
 	 */
 	spgPageIndexMultiDelete(state, current->page, toDelete, nDelete,
-					   state->isBuild ? SPGIST_PLACEHOLDER : SPGIST_REDIRECT,
+							state->isBuild ? SPGIST_PLACEHOLDER : SPGIST_REDIRECT,
 							SPGIST_PLACEHOLDER,
 							nblkno, r);
 
@@ -569,7 +570,7 @@ setRedirectionTuple(SPPageDesc *current, OffsetNumber position,
 	SpGistDeadTuple dt;
 
 	dt = (SpGistDeadTuple) PageGetItem(current->page,
-									 PageGetItemId(current->page, position));
+									   PageGetItemId(current->page, position));
 	Assert(dt->tupstate == SPGIST_REDIRECT);
 	Assert(ItemPointerGetBlockNumber(&dt->pointer) == SPGIST_METAPAGE_BLKNO);
 	ItemPointerSet(&dt->pointer, blkno, offnum);
@@ -579,7 +580,7 @@ setRedirectionTuple(SPPageDesc *current, OffsetNumber position,
  * Test to see if the user-defined picksplit function failed to do its job,
  * ie, it put all the leaf tuples into the same node.
  * If so, randomly divide the tuples into several nodes (all with the same
- * label) and return TRUE to select allTheSame mode for this inner tuple.
+ * label) and return true to select allTheSame mode for this inner tuple.
  *
  * (This code is also used to forcibly select allTheSame mode for nulls.)
  *
@@ -668,8 +669,7 @@ checkAllTheSame(spgPickSplitIn *in, spgPickSplitOut *out, bool tooBig,
  * will eventually terminate if lack of balance is the issue.  If the tuple
  * is too big, we assume that repeated picksplit operations will eventually
  * make it small enough by repeated prefix-stripping.  A broken opclass could
- * make this an infinite loop, though, so spgdoinsert() checks that the
- * leaf datums get smaller each time.
+ * make this an infinite loop, though.
  */
 static bool
 doPickSplit(Relation index, SpGistState *state,
@@ -754,7 +754,7 @@ doPickSplit(Relation index, SpGistState *state,
 			SpGistLeafTuple it;
 
 			it = (SpGistLeafTuple) PageGetItem(current->page,
-											PageGetItemId(current->page, i));
+											   PageGetItemId(current->page, i));
 			if (it->tupstate == SPGIST_LIVE)
 			{
 				in.datums[nToInsert] = SGLTDATUM(it, state);
@@ -765,7 +765,7 @@ doPickSplit(Relation index, SpGistState *state,
 				/* we will delete the tuple altogether, so count full space */
 				spaceToDelete += it->size + sizeof(ItemIdData);
 			}
-			else	/* tuples on root should be live */
+			else				/* tuples on root should be live */
 				elog(ERROR, "unexpected SPGiST tuple state: %d", it->tupstate);
 		}
 	}
@@ -779,7 +779,7 @@ doPickSplit(Relation index, SpGistState *state,
 
 			Assert(i >= FirstOffsetNumber && i <= max);
 			it = (SpGistLeafTuple) PageGetItem(current->page,
-											PageGetItemId(current->page, i));
+											   PageGetItemId(current->page, i));
 			if (it->tupstate == SPGIST_LIVE)
 			{
 				in.datums[nToInsert] = SGLTDATUM(it, state);
@@ -957,9 +957,9 @@ doPickSplit(Relation index, SpGistState *state,
 	{
 		/* Send tuple to page with next triple parity (see README) */
 		newInnerBuffer = SpGistGetBuffer(index,
-									   GBUF_INNER_PARITY(parent->blkno + 1) |
+										 GBUF_INNER_PARITY(parent->blkno + 1) |
 										 (isNulls ? GBUF_NULLS : 0),
-									   innerTuple->size + sizeof(ItemIdData),
+										 innerTuple->size + sizeof(ItemIdData),
 										 &xlrec.initInner);
 	}
 	else
@@ -1004,7 +1004,7 @@ doPickSplit(Relation index, SpGistState *state,
 			insertedNew = true;
 		}
 		for (i = 0; i < nToInsert; i++)
-			leafPageSelect[i] = 0;		/* signifies current page */
+			leafPageSelect[i] = 0;	/* signifies current page */
 	}
 	else if (in.nTuples == 1 && totalLeafSizes > SPGIST_PAGE_CAPACITY)
 	{
@@ -1025,7 +1025,7 @@ doPickSplit(Relation index, SpGistState *state,
 		int			newspace;
 
 		newLeafBuffer = SpGistGetBuffer(index,
-									  GBUF_LEAF | (isNulls ? GBUF_NULLS : 0),
+										GBUF_LEAF | (isNulls ? GBUF_NULLS : 0),
 										Min(totalLeafSizes,
 											SPGIST_PAGE_CAPACITY),
 										&xlrec.initDest);
@@ -1076,12 +1076,12 @@ doPickSplit(Relation index, SpGistState *state,
 			{
 				if (leafSizes[i] <= curspace)
 				{
-					nodePageSelect[i] = 0;		/* signifies current page */
+					nodePageSelect[i] = 0;	/* signifies current page */
 					curspace -= leafSizes[i];
 				}
 				else
 				{
-					nodePageSelect[i] = 1;		/* signifies new leaf page */
+					nodePageSelect[i] = 1;	/* signifies new leaf page */
 					newspace -= leafSizes[i];
 				}
 			}
@@ -1576,7 +1576,7 @@ spgAddNodeAction(Relation index, SpGistState *state,
 		 */
 		current->buffer = SpGistGetBuffer(index,
 										  GBUF_INNER_PARITY(current->blkno),
-									newInnerTuple->size + sizeof(ItemIdData),
+										  newInnerTuple->size + sizeof(ItemIdData),
 										  &xlrec.newPage);
 		current->blkno = BufferGetBlockNumber(current->buffer);
 		current->page = BufferGetPage(current->buffer);
@@ -1706,17 +1706,40 @@ spgSplitNodeAction(Relation index, SpGistState *state,
 	/* Should not be applied to nulls */
 	Assert(!SpGistPageStoresNulls(current->page));
 
+	/* Check opclass gave us sane values */
+	if (out->result.splitTuple.prefixNNodes <= 0 ||
+		out->result.splitTuple.prefixNNodes > SGITMAXNNODES)
+		elog(ERROR, "invalid number of prefix nodes: %d",
+			 out->result.splitTuple.prefixNNodes);
+	if (out->result.splitTuple.childNodeN < 0 ||
+		out->result.splitTuple.childNodeN >=
+		out->result.splitTuple.prefixNNodes)
+		elog(ERROR, "invalid child node number: %d",
+			 out->result.splitTuple.childNodeN);
+
 	/*
-	 * Construct new prefix tuple, containing a single node with the specified
-	 * label.  (We'll update the node's downlink to point to the new postfix
-	 * tuple, below.)
+	 * Construct new prefix tuple with requested number of nodes.  We'll fill
+	 * in the childNodeN'th node's downlink below.
 	 */
-	node = spgFormNodeTuple(state, out->result.splitTuple.nodeLabel, false);
+	nodes = (SpGistNodeTuple *) palloc(sizeof(SpGistNodeTuple) *
+									   out->result.splitTuple.prefixNNodes);
+
+	for (i = 0; i < out->result.splitTuple.prefixNNodes; i++)
+	{
+		Datum		label = (Datum) 0;
+		bool		labelisnull;
+
+		labelisnull = (out->result.splitTuple.prefixNodeLabels == NULL);
+		if (!labelisnull)
+			label = out->result.splitTuple.prefixNodeLabels[i];
+		nodes[i] = spgFormNodeTuple(state, label, labelisnull);
+	}
 
 	prefixTuple = spgFormInnerTuple(state,
 									out->result.splitTuple.prefixHasPrefix,
 									out->result.splitTuple.prefixPrefixDatum,
-									1, &node);
+									out->result.splitTuple.prefixNNodes,
+									nodes);
 
 	/* it must fit in the space that innerTuple now occupies */
 	if (prefixTuple->size > innerTuple->size)
@@ -1735,7 +1758,7 @@ spgSplitNodeAction(Relation index, SpGistState *state,
 
 	postfixTuple = spgFormInnerTuple(state,
 									 out->result.splitTuple.postfixHasPrefix,
-								   out->result.splitTuple.postfixPrefixDatum,
+									 out->result.splitTuple.postfixPrefixDatum,
 									 innerTuple->nNodes, nodes);
 
 	/* Postfix tuple is allTheSame if original tuple was */
@@ -1808,10 +1831,12 @@ spgSplitNodeAction(Relation index, SpGistState *state,
 	 * the postfix tuple first.)  We have to update the local copy of the
 	 * prefixTuple too, because that's what will be written to WAL.
 	 */
-	spgUpdateNodeLink(prefixTuple, 0, postfixBlkno, postfixOffset);
+	spgUpdateNodeLink(prefixTuple, out->result.splitTuple.childNodeN,
+					  postfixBlkno, postfixOffset);
 	prefixTuple = (SpGistInnerTuple) PageGetItem(current->page,
-							  PageGetItemId(current->page, current->offnum));
-	spgUpdateNodeLink(prefixTuple, 0, postfixBlkno, postfixOffset);
+												 PageGetItemId(current->page, current->offnum));
+	spgUpdateNodeLink(prefixTuple, out->result.splitTuple.childNodeN,
+					  postfixBlkno, postfixOffset);
 
 	MarkBufferDirty(current->buffer);
 
@@ -1859,19 +1884,16 @@ spgSplitNodeAction(Relation index, SpGistState *state,
  * Insert one item into the index.
  *
  * Returns true on success, false if we failed to complete the insertion
- * (typically because of conflict with a concurrent insert).  In the latter
- * case, caller should re-call spgdoinsert() with the same args.
+ * because of conflict with a concurrent insert.  In the latter case,
+ * caller should re-call spgdoinsert() with the same args.
  */
 bool
 spgdoinsert(Relation index, SpGistState *state,
 			ItemPointer heapPtr, Datum datum, bool isnull)
 {
-	bool		result = true;
 	int			level = 0;
 	Datum		leafDatum;
 	int			leafSize;
-	int			bestLeafSize;
-	int			numNoProgressCycles = 0;
 	SPPageDesc	current,
 				parent;
 	FmgrInfo   *procinfo = NULL;
@@ -1884,37 +1906,59 @@ spgdoinsert(Relation index, SpGistState *state,
 		procinfo = index_getprocinfo(index, 1, SPGIST_CHOOSE_PROC);
 
 	/*
-	 * Since we don't use index_form_tuple in this AM, we have to make sure
-	 * value to be inserted is not toasted; FormIndexDatum doesn't guarantee
-	 * that.
+	 * Prepare the leaf datum to insert.
+	 *
+	 * If an optional "compress" method is provided, then call it to form the
+	 * leaf datum from the input datum.  Otherwise store the input datum as
+	 * is.  Since we don't use index_form_tuple in this AM, we have to make
+	 * sure value to be inserted is not toasted; FormIndexDatum doesn't
+	 * guarantee that.  But we assume the "compress" method to return an
+	 * untoasted value.
 	 */
-	if (!isnull && state->attType.attlen == -1)
-		datum = PointerGetDatum(PG_DETOAST_DATUM(datum));
+	if (!isnull)
+	{
+		if (OidIsValid(index_getprocid(index, 1, SPGIST_COMPRESS_PROC)))
+		{
+			FmgrInfo   *compressProcinfo = NULL;
 
-	leafDatum = datum;
+			compressProcinfo = index_getprocinfo(index, 1, SPGIST_COMPRESS_PROC);
+			leafDatum = FunctionCall1Coll(compressProcinfo,
+										  index->rd_indcollation[0],
+										  datum);
+		}
+		else
+		{
+			Assert(state->attLeafType.type == state->attType.type);
+
+			if (state->attType.attlen == -1)
+				leafDatum = PointerGetDatum(PG_DETOAST_DATUM(datum));
+			else
+				leafDatum = datum;
+		}
+	}
+	else
+		leafDatum = (Datum) 0;
 
 	/*
 	 * Compute space needed for a leaf tuple containing the given datum.
 	 *
 	 * If it isn't gonna fit, and the opclass can't reduce the datum size by
-	 * suffixing, bail out now rather than doing a lot of useless work.
+	 * suffixing, bail out now rather than getting into an endless loop.
 	 */
 	if (!isnull)
 		leafSize = SGLTHDRSZ + sizeof(ItemIdData) +
-			SpGistGetTypeSize(&state->attType, leafDatum);
+			SpGistGetTypeSize(&state->attLeafType, leafDatum);
 	else
 		leafSize = SGDTSIZE + sizeof(ItemIdData);
 
-	if (leafSize > SPGIST_PAGE_CAPACITY &&
-		(isnull || !state->config.longValuesOK))
+	if (leafSize > SPGIST_PAGE_CAPACITY && !state->config.longValuesOK)
 		ereport(ERROR,
 				(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
-			errmsg("index row size %zu exceeds maximum %zu for index \"%s\"",
-				   leafSize - sizeof(ItemIdData),
-				   SPGIST_PAGE_CAPACITY - sizeof(ItemIdData),
-				   RelationGetRelationName(index)),
-			errhint("Values larger than a buffer page cannot be indexed.")));
-	bestLeafSize = leafSize;
+				 errmsg("index row size %zu exceeds maximum %zu for index \"%s\"",
+						leafSize - sizeof(ItemIdData),
+						SPGIST_PAGE_CAPACITY - sizeof(ItemIdData),
+						RelationGetRelationName(index)),
+				 errhint("Values larger than a buffer page cannot be indexed.")));
 
 	/* Initialize "current" to the appropriate root page */
 	current.blkno = isnull ? SPGIST_NULL_BLKNO : SPGIST_ROOT_BLKNO;
@@ -1930,14 +1974,6 @@ spgdoinsert(Relation index, SpGistState *state,
 	parent.offnum = InvalidOffsetNumber;
 	parent.node = -1;
 
-	/*
-	 * Before entering the loop, try to clear any pending interrupt condition.
-	 * If a query cancel is pending, we might as well accept it now not later;
-	 * while if a non-canceling condition is pending, servicing it here avoids
-	 * having to restart the insertion and redo all the work so far.
-	 */
-	CHECK_FOR_INTERRUPTS();
-
 	for (;;)
 	{
 		bool		isNew = false;
@@ -1945,18 +1981,9 @@ spgdoinsert(Relation index, SpGistState *state,
 		/*
 		 * Bail out if query cancel is pending.  We must have this somewhere
 		 * in the loop since a broken opclass could produce an infinite
-		 * picksplit loop.  However, because we'll be holding buffer lock(s)
-		 * after the first iteration, ProcessInterrupts() wouldn't be able to
-		 * throw a cancel error here.  Hence, if we see that an interrupt is
-		 * pending, break out of the loop and deal with the situation below.
-		 * Set result = false because we must restart the insertion if the
-		 * interrupt isn't a query-cancel-or-die case.
+		 * picksplit loop.
 		 */
-		if (INTERRUPTS_PENDING_CONDITION())
-		{
-			result = false;
-			break;
-		}
+		CHECK_FOR_INTERRUPTS();
 
 		if (current.blkno == InvalidBlockNumber)
 		{
@@ -2032,7 +2059,7 @@ spgdoinsert(Relation index, SpGistState *state,
 			}
 			else if ((sizeToSplit =
 					  checkSplitConditions(index, state, &current,
-									&nToSplit)) < SPGIST_PAGE_CAPACITY / 2 &&
+										   &nToSplit)) < SPGIST_PAGE_CAPACITY / 2 &&
 					 nToSplit < 64 &&
 					 leafTuple->size + sizeof(ItemIdData) + sizeToSplit <= SPGIST_PAGE_CAPACITY)
 			{
@@ -2061,7 +2088,7 @@ spgdoinsert(Relation index, SpGistState *state,
 				goto process_inner_tuple;
 			}
 		}
-		else	/* non-leaf page */
+		else					/* non-leaf page */
 		{
 			/*
 			 * Apply the opclass choose function to figure out how to insert
@@ -2075,17 +2102,13 @@ spgdoinsert(Relation index, SpGistState *state,
 			 * spgAddNode and spgSplitTuple cases will loop back to here to
 			 * complete the insertion operation.  Just in case the choose
 			 * function is broken and produces add or split requests
-			 * repeatedly, check for query cancel (see comments above).
+			 * repeatedly, check for query cancel.
 			 */
 	process_inner_tuple:
-			if (INTERRUPTS_PENDING_CONDITION())
-			{
-				result = false;
-				break;
-			}
+			CHECK_FOR_INTERRUPTS();
 
 			innerTuple = (SpGistInnerTuple) PageGetItem(current.page,
-								PageGetItemId(current.page, current.offnum));
+														PageGetItemId(current.page, current.offnum));
 
 			in.datum = datum;
 			in.leafDatum = leafDatum;
@@ -2139,49 +2162,7 @@ spgdoinsert(Relation index, SpGistState *state,
 					{
 						leafDatum = out.result.matchNode.restDatum;
 						leafSize = SGLTHDRSZ + sizeof(ItemIdData) +
-							SpGistGetTypeSize(&state->attType, leafDatum);
-					}
-
-					/*
-					 * Check new tuple size; fail if it can't fit, unless the
-					 * opclass says it can handle the situation by suffixing.
-					 *
-					 * A buggy opclass might not ever make the leaf datum
-					 * small enough, causing an infinite loop.  To detect such
-					 * a loop, check to see if we are making progress by
-					 * reducing the leafSize in each pass.  This is a bit
-					 * tricky though.  Because of alignment considerations,
-					 * the total tuple size might not decrease on every pass.
-					 * Also, there are edge cases where the choose method
-					 * might seem to not make progress for a cycle or two.
-					 * Somewhat arbitrarily, we allow up to 10 no-progress
-					 * iterations before failing.  (This limit should be more
-					 * than MAXALIGN, to accommodate opclasses that trim one
-					 * byte from the leaf datum per pass.)
-					 */
-					if (leafSize > SPGIST_PAGE_CAPACITY)
-					{
-						bool		ok = false;
-
-						if (state->config.longValuesOK && !isnull)
-						{
-							if (leafSize < bestLeafSize)
-							{
-								ok = true;
-								bestLeafSize = leafSize;
-								numNoProgressCycles = 0;
-							}
-							else if (++numNoProgressCycles < 10)
-								ok = true;
-						}
-						if (!ok)
-							ereport(ERROR,
-									(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
-									 errmsg("index row size %zu exceeds maximum %zu for index \"%s\"",
-											leafSize - sizeof(ItemIdData),
-											SPGIST_PAGE_CAPACITY - sizeof(ItemIdData),
-											RelationGetRelationName(index)),
-									 errhint("Values larger than a buffer page cannot be indexed.")));
+							SpGistGetTypeSize(&state->attLeafType, leafDatum);
 					}
 
 					/*
@@ -2189,6 +2170,14 @@ spgdoinsert(Relation index, SpGistState *state,
 					 * "current" (which might reference an existing child
 					 * tuple, or might be invalid to force us to find a new
 					 * page for the tuple).
+					 *
+					 * Note: if the opclass sets longValuesOK, we rely on the
+					 * choose function to eventually shorten the leafDatum
+					 * enough to fit on a page.  We could add a test here to
+					 * complain if the datum doesn't get visibly shorter each
+					 * time, but that could get in the way of opclasses that
+					 * "simplify" datums in a way that doesn't necessarily
+					 * lead to physical shortening on every cycle.
 					 */
 					break;
 				case spgAddNode:
@@ -2239,21 +2228,5 @@ spgdoinsert(Relation index, SpGistState *state,
 		UnlockReleaseBuffer(parent.buffer);
 	}
 
-	/*
-	 * We do not support being called while some outer function is holding a
-	 * buffer lock (or any other reason to postpone query cancels).  If that
-	 * were the case, telling the caller to retry would create an infinite
-	 * loop.
-	 */
-	Assert(INTERRUPTS_CAN_BE_PROCESSED());
-
-	/*
-	 * Finally, check for interrupts again.  If there was a query cancel,
-	 * ProcessInterrupts() will be able to throw the error here.  If it was
-	 * some other kind of interrupt that can just be cleared, return false to
-	 * tell our caller to retry.
-	 */
-	CHECK_FOR_INTERRUPTS();
-
-	return result;
+	return true;
 }
