@@ -224,11 +224,6 @@ CreatePublication(CreatePublicationStmt *stmt)
 		PublicationAddTables(puboid, rels, true, NULL);
 		CloseTableList(rels);
 	}
-	else if (stmt->for_all_tables)
-	{
-		/* Invalidate relcache so that publication info is rebuilt. */
-		CacheInvalidateRelcacheAll();
-	}
 
 	heap_close(rel, RowExclusiveLock);
 
@@ -443,7 +438,6 @@ RemovePublicationById(Oid pubid)
 {
 	Relation	rel;
 	HeapTuple	tup;
-	Form_pg_publication pubform;
 
 	rel = heap_open(PublicationRelationId, RowExclusiveLock);
 
@@ -452,12 +446,6 @@ RemovePublicationById(Oid pubid)
 	if (!HeapTupleIsValid(tup))
 		elog(ERROR, "cache lookup failed for publication %u", pubid);
 
-	pubform = (Form_pg_publication) GETSTRUCT(tup);
-
-	/* Invalidate relcache so that publication info is rebuilt. */
-	if (pubform->puballtables)
-		CacheInvalidateRelcacheAll();
-
 	CatalogTupleDelete(rel, &tup->t_self);
 
 	ReleaseSysCache(tup);
