@@ -256,8 +256,12 @@ SPI_commit(void)
 	/* Start the actual commit */
 	_SPI_current->internal_xact = true;
 
-	/* Release snapshots associated with portals */
-	ForgetPortalSnapshots();
+	/*
+	 * Before committing, pop all active snapshots to avoid error about
+	 * "snapshot %p still active".
+	 */
+	while (ActiveSnapshotSet())
+		PopActiveSnapshot();
 
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(oldcontext);
@@ -292,9 +296,6 @@ SPI_rollback(void)
 	/* Start the actual rollback */
 	_SPI_current->internal_xact = true;
 
-	/* Release snapshots associated with portals */
-	ForgetPortalSnapshots();
-
 	AbortCurrentTransaction();
 	MemoryContextSwitchTo(oldcontext);
 
@@ -2070,7 +2071,6 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 	Oid			my_lastoid = InvalidOid;
 	SPITupleTable *my_tuptable = NULL;
 	int			res = 0;
-	bool		allow_nonatomic = plan->no_snapshots;	/* legacy API name */
 	bool		pushed_active_snap = false;
 	ErrorContextCallback spierrcontext;
 	CachedPlan *cplan = NULL;
@@ -2103,12 +2103,11 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 	 * In the first two cases, we can just push the snap onto the stack once
 	 * for the whole plan list.
 	 *
-	 * Note that snapshot != InvalidSnapshot implies an atomic execution
-	 * context.
+	 * But if the plan has no_snapshots set to true, then don't manage
+	 * snapshots at all.  The caller should then take care of that.
 	 */
-	if (snapshot != InvalidSnapshot)
+	if (snapshot != InvalidSnapshot && !plan->no_snapshots)
 	{
-		Assert(!allow_nonatomic);
 		if (read_only)
 		{
 			PushActiveSnapshot(snapshot);
@@ -2183,39 +2182,15 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 		stmt_list = cplan->stmt_list;
 
 		/*
-		 * If we weren't given a specific snapshot to use, and the statement
-		 * list requires a snapshot, set that up.
+		 * In the default non-read-only case, get a new snapshot, replacing
+		 * any that we pushed in a previous cycle.
 		 */
-		if (snapshot == InvalidSnapshot &&
-			(list_length(stmt_list) > 1 ||
-			 (list_length(stmt_list) == 1 &&
-			  PlannedStmtRequiresSnapshot(linitial_node(PlannedStmt,
-														stmt_list)))))
+		if (snapshot == InvalidSnapshot && !read_only && !plan->no_snapshots)
 		{
-			/*
-			 * First, ensure there's a Portal-level snapshot.  This back-fills
-			 * the snapshot stack in case the previous operation was a COMMIT
-			 * or ROLLBACK inside a procedure or DO block.  (We can't put back
-			 * the Portal snapshot any sooner, or we'd break cases like doing
-			 * SET or LOCK just after COMMIT.)  It's enough to check once per
-			 * statement list, since COMMIT/ROLLBACK/CALL/DO can't appear
-			 * within a multi-statement list.
-			 */
-			EnsurePortalSnapshotExists();
-
-			/*
-			 * In the default non-read-only case, get a new per-statement-list
-			 * snapshot, replacing any that we pushed in a previous cycle.
-			 * Skip it when doing non-atomic execution, though (we rely
-			 * entirely on the Portal snapshot in that case).
-			 */
-			if (!read_only && !allow_nonatomic)
-			{
-				if (pushed_active_snap)
-					PopActiveSnapshot();
-				PushActiveSnapshot(GetTransactionSnapshot());
-				pushed_active_snap = true;
-			}
+			if (pushed_active_snap)
+				PopActiveSnapshot();
+			PushActiveSnapshot(GetTransactionSnapshot());
+			pushed_active_snap = true;
 		}
 
 		foreach(lc2, stmt_list)
@@ -2228,7 +2203,6 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 			_SPI_current->lastoid = InvalidOid;
 			_SPI_current->tuptable = NULL;
 
-			/* Check for unsupported cases. */
 			if (stmt->utilityStmt)
 			{
 				if (IsA(stmt->utilityStmt, CopyStmt))
@@ -2260,10 +2234,9 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 
 			/*
 			 * If not read-only mode, advance the command counter before each
-			 * command and update the snapshot.  (But skip it if the snapshot
-			 * isn't under our control.)
+			 * command and update the snapshot.
 			 */
-			if (!read_only && pushed_active_snap)
+			if (!read_only && !plan->no_snapshots)
 			{
 				CommandCounterIncrement();
 				UpdateActiveSnapshotCommandId();
@@ -2297,11 +2270,13 @@ _SPI_execute_plan(SPIPlanPtr plan, ParamListInfo paramLI,
 				ProcessUtilityContext context;
 
 				/*
-				 * If the SPI context is atomic, or we were not told to allow
-				 * nonatomic operations, tell ProcessUtility this is an atomic
-				 * execution context.
+				 * If the SPI context is atomic, or we are asked to manage
+				 * snapshots, then we are in an atomic execution context.
+				 * Conversely, to propagate a nonatomic execution context, the
+				 * caller must be in a nonatomic SPI context and manage
+				 * snapshots itself.
 				 */
-				if (_SPI_current->atomic || !allow_nonatomic)
+				if (_SPI_current->atomic || !plan->no_snapshots)
 					context = PROCESS_UTILITY_QUERY;
 				else
 					context = PROCESS_UTILITY_QUERY_NONATOMIC;
