@@ -1019,26 +1019,8 @@ set_indexonlyscan_references(PlannerInfo *root,
 							 int rtoffset)
 {
 	indexed_tlist *index_itlist;
-	List	   *stripped_indextlist;
-	ListCell   *lc;
 
-	/*
-	 * Vars in the plan node's targetlist, qual, and recheckqual must only
-	 * reference columns that the index AM can actually return.  To ensure
-	 * this, remove non-returnable columns (which are marked as resjunk) from
-	 * the indexed tlist.  We can just drop them because the indexed_tlist
-	 * machinery pays attention to TLE resnos, not physical list position.
-	 */
-	stripped_indextlist = NIL;
-	foreach(lc, plan->indextlist)
-	{
-		TargetEntry *indextle = (TargetEntry *) lfirst(lc);
-
-		if (!indextle->resjunk)
-			stripped_indextlist = lappend(stripped_indextlist, indextle);
-	}
-
-	index_itlist = build_tlist_index(stripped_indextlist);
+	index_itlist = build_tlist_index(plan->indextlist);
 
 	plan->scan.scanrelid += rtoffset;
 	plan->scan.plan.targetlist = (List *)
@@ -1053,12 +1035,6 @@ set_indexonlyscan_references(PlannerInfo *root,
 					   index_itlist,
 					   INDEX_VAR,
 					   rtoffset);
-	plan->recheckqual = (List *)
-		fix_upper_expr(root,
-					   (Node *) plan->recheckqual,
-					   index_itlist,
-					   INDEX_VAR,
-					   rtoffset);
 	/* indexqual is already transformed to reference index columns */
 	plan->indexqual = fix_scan_list(root, plan->indexqual, rtoffset);
 	/* indexorderby is already transformed to reference index columns */
@@ -1102,14 +1078,7 @@ set_subqueryscan_references(PlannerInfo *root,
 
 		result = plan->subplan;
 
-		/*
-		 * We have to be sure we don't lose any initplans, so move any that
-		 * were attached to the parent plan to the child.  If we do move any,
-		 * the child is no longer parallel-safe.
-		 */
-		if (plan->scan.plan.initPlan)
-			result->parallel_safe = false;
-
+		/* We have to be sure we don't lose any initplans */
 		result->initPlan = list_concat(plan->scan.plan.initPlan,
 									   result->initPlan);
 
@@ -2649,25 +2618,8 @@ extract_query_dependencies_walker(Node *node, PlannerInfo *context)
 		if (query->commandType == CMD_UTILITY)
 		{
 			/*
-			 * This logic must handle any utility command for which parse
-			 * analysis was nontrivial (cf. stmt_requires_parse_analysis).
-			 *
-			 * Notably, CALL requires its own processing.
-			 */
-			if (IsA(query->utilityStmt, CallStmt))
-			{
-				CallStmt   *callstmt = (CallStmt *) query->utilityStmt;
-
-				/* We need not examine funccall, just the transformed exprs */
-				(void) extract_query_dependencies_walker((Node *) callstmt->funcexpr,
-														 context);
-				return false;
-			}
-
-			/*
-			 * Ignore other utility statements, except those (such as EXPLAIN)
-			 * that contain a parsed-but-not-planned query.  For those, we
-			 * just need to transfer our attention to the contained query.
+			 * Ignore utility statements, except those (such as EXPLAIN) that
+			 * contain a parsed-but-not-planned query.
 			 */
 			query = UtilityContainsQuery(query->utilityStmt);
 			if (query == NULL)
