@@ -161,7 +161,10 @@ libpqrcv_connect(const char *conninfo, bool logical, const char *appname,
 	conn->streamConn = PQconnectStartParams(keys, vals,
 											 /* expand_dbname = */ true);
 	if (PQstatus(conn->streamConn) == CONNECTION_BAD)
-		goto bad_connection_errmsg;
+	{
+		*err = pchomp(PQerrorMessage(conn->streamConn));
+		return NULL;
+	}
 
 	/*
 	 * Poll connection until we have OK or FAILED status.
@@ -208,7 +211,10 @@ libpqrcv_connect(const char *conninfo, bool logical, const char *appname,
 	} while (status != PGRES_POLLING_OK && status != PGRES_POLLING_FAILED);
 
 	if (PQstatus(conn->streamConn) != CONNECTION_OK)
-		goto bad_connection_errmsg;
+	{
+		*err = pchomp(PQerrorMessage(conn->streamConn));
+		return NULL;
+	}
 
 	if (logical)
 	{
@@ -219,9 +225,9 @@ libpqrcv_connect(const char *conninfo, bool logical, const char *appname,
 		if (PQresultStatus(res) != PGRES_TUPLES_OK)
 		{
 			PQclear(res);
-			*err = psprintf(_("could not clear search path: %s"),
-							pchomp(PQerrorMessage(conn->streamConn)));
-			goto bad_connection;
+			ereport(ERROR,
+					(errmsg("could not clear search path: %s",
+							pchomp(PQerrorMessage(conn->streamConn)))));
 		}
 		PQclear(res);
 	}
@@ -229,16 +235,6 @@ libpqrcv_connect(const char *conninfo, bool logical, const char *appname,
 	conn->logical = logical;
 
 	return conn;
-
-	/* error path, using libpq's error message */
-bad_connection_errmsg:
-	*err = pchomp(PQerrorMessage(conn->streamConn));
-
-	/* error path, error already set */
-bad_connection:
-	PQfinish(conn->streamConn);
-	pfree(conn);
-	return NULL;
 }
 
 /*
