@@ -3,7 +3,7 @@
  * plpgsql.h		- Definitions for the PL/pgSQL
  *			  procedural language
  *
- * Portions Copyright (c) 1996-2016, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -16,12 +16,13 @@
 #ifndef PLPGSQL_H
 #define PLPGSQL_H
 
-#include "postgres.h"
-
 #include "access/xact.h"
 #include "commands/event_trigger.h"
 #include "commands/trigger.h"
 #include "executor/spi.h"
+#include "utils/expandedrecord.h"
+#include "utils/typcache.h"
+
 
 /**********************************************************************
  * Definitions
@@ -34,60 +35,72 @@
 #undef _
 #define _(x) dgettext(TEXTDOMAIN, x)
 
-/* ----------
+/*
  * Compiler's namespace item types
- * ----------
  */
-enum
+typedef enum PLpgSQL_nsitem_type
 {
-	PLPGSQL_NSTYPE_LABEL,
-	PLPGSQL_NSTYPE_VAR,
-	PLPGSQL_NSTYPE_ROW,
-	PLPGSQL_NSTYPE_REC
-};
+	PLPGSQL_NSTYPE_LABEL,		/* block label */
+	PLPGSQL_NSTYPE_VAR,			/* scalar variable */
+	PLPGSQL_NSTYPE_REC			/* composite variable */
+} PLpgSQL_nsitem_type;
 
-/* ----------
+/*
  * A PLPGSQL_NSTYPE_LABEL stack entry must be one of these types
- * ----------
  */
-enum PLpgSQL_label_types
+typedef enum PLpgSQL_label_type
 {
 	PLPGSQL_LABEL_BLOCK,		/* DECLARE/BEGIN block */
 	PLPGSQL_LABEL_LOOP,			/* looping construct */
 	PLPGSQL_LABEL_OTHER			/* anything else */
-};
+} PLpgSQL_label_type;
 
-/* ----------
+/*
  * Datum array node types
- * ----------
  */
-enum
+typedef enum PLpgSQL_datum_type
 {
 	PLPGSQL_DTYPE_VAR,
 	PLPGSQL_DTYPE_ROW,
 	PLPGSQL_DTYPE_REC,
 	PLPGSQL_DTYPE_RECFIELD,
 	PLPGSQL_DTYPE_ARRAYELEM,
-	PLPGSQL_DTYPE_EXPR
-};
+	PLPGSQL_DTYPE_PROMISE
+} PLpgSQL_datum_type;
 
-/* ----------
- * Variants distinguished in PLpgSQL_type structs
- * ----------
+/*
+ * DTYPE_PROMISE datums have these possible ways of computing the promise
  */
-enum
+typedef enum PLpgSQL_promise_type
+{
+	PLPGSQL_PROMISE_NONE = 0,	/* not a promise, or promise satisfied */
+	PLPGSQL_PROMISE_TG_NAME,
+	PLPGSQL_PROMISE_TG_WHEN,
+	PLPGSQL_PROMISE_TG_LEVEL,
+	PLPGSQL_PROMISE_TG_OP,
+	PLPGSQL_PROMISE_TG_RELID,
+	PLPGSQL_PROMISE_TG_TABLE_NAME,
+	PLPGSQL_PROMISE_TG_TABLE_SCHEMA,
+	PLPGSQL_PROMISE_TG_NARGS,
+	PLPGSQL_PROMISE_TG_ARGV,
+	PLPGSQL_PROMISE_TG_EVENT,
+	PLPGSQL_PROMISE_TG_TAG
+} PLpgSQL_promise_type;
+
+/*
+ * Variants distinguished in PLpgSQL_type structs
+ */
+typedef enum PLpgSQL_type_type
 {
 	PLPGSQL_TTYPE_SCALAR,		/* scalar types and domains */
-	PLPGSQL_TTYPE_ROW,			/* composite types */
-	PLPGSQL_TTYPE_REC,			/* RECORD pseudotype */
-	PLPGSQL_TTYPE_PSEUDO		/* other pseudotypes */
-};
+	PLPGSQL_TTYPE_REC,			/* composite types, including RECORD */
+	PLPGSQL_TTYPE_PSEUDO		/* pseudotypes */
+} PLpgSQL_type_type;
 
-/* ----------
+/*
  * Execution tree node types
- * ----------
  */
-enum PLpgSQL_stmt_types
+typedef enum PLpgSQL_stmt_type
 {
 	PLPGSQL_STMT_BLOCK,
 	PLPGSQL_STMT_ASSIGN,
@@ -112,13 +125,15 @@ enum PLpgSQL_stmt_types
 	PLPGSQL_STMT_OPEN,
 	PLPGSQL_STMT_FETCH,
 	PLPGSQL_STMT_CLOSE,
-	PLPGSQL_STMT_PERFORM
-};
+	PLPGSQL_STMT_PERFORM,
+	PLPGSQL_STMT_CALL,
+	PLPGSQL_STMT_COMMIT,
+	PLPGSQL_STMT_ROLLBACK,
+	PLPGSQL_STMT_SET
+} PLpgSQL_stmt_type;
 
-
-/* ----------
+/*
  * Execution node return codes
- * ----------
  */
 enum
 {
@@ -128,11 +143,10 @@ enum
 	PLPGSQL_RC_CONTINUE
 };
 
-/* ----------
+/*
  * GET DIAGNOSTICS information items
- * ----------
  */
-enum
+typedef enum PLpgSQL_getdiag_kind
 {
 	PLPGSQL_GETDIAG_ROW_COUNT,
 	PLPGSQL_GETDIAG_RESULT_OID,
@@ -147,13 +161,12 @@ enum
 	PLPGSQL_GETDIAG_MESSAGE_TEXT,
 	PLPGSQL_GETDIAG_TABLE_NAME,
 	PLPGSQL_GETDIAG_SCHEMA_NAME
-};
+} PLpgSQL_getdiag_kind;
 
-/* --------
+/*
  * RAISE statement options
- * --------
  */
-enum
+typedef enum PLpgSQL_raise_option_type
 {
 	PLPGSQL_RAISEOPTION_ERRCODE,
 	PLPGSQL_RAISEOPTION_MESSAGE,
@@ -164,13 +177,12 @@ enum
 	PLPGSQL_RAISEOPTION_DATATYPE,
 	PLPGSQL_RAISEOPTION_TABLE,
 	PLPGSQL_RAISEOPTION_SCHEMA
-};
+} PLpgSQL_raise_option_type;
 
-/* --------
+/*
  * Behavioral modes for plpgsql variable resolution
- * --------
  */
-typedef enum
+typedef enum PLpgSQL_resolve_option
 {
 	PLPGSQL_RESOLVE_ERROR,		/* throw error if ambiguous */
 	PLPGSQL_RESOLVE_VARIABLE,	/* prefer plpgsql var to table column */
@@ -182,48 +194,31 @@ typedef enum
  * Node and structure definitions
  **********************************************************************/
 
-
-typedef struct
-{								/* Postgres data type */
+/*
+ * Postgres data type
+ */
+typedef struct PLpgSQL_type
+{
 	char	   *typname;		/* (simple) name of the type */
 	Oid			typoid;			/* OID of the data type */
-	int			ttype;			/* PLPGSQL_TTYPE_ code */
+	PLpgSQL_type_type ttype;	/* PLPGSQL_TTYPE_ code */
 	int16		typlen;			/* stuff copied from its pg_type entry */
 	bool		typbyval;
 	char		typtype;
-	Oid			typrelid;
 	Oid			collation;		/* from pg_type, but can be overridden */
 	bool		typisarray;		/* is "true" array, or domain over one */
 	int32		atttypmod;		/* typmod (taken from someplace else) */
+	/* Remaining fields are used only for named composite types (not RECORD) */
+	TypeName   *origtypname;	/* type name as written by user */
+	TypeCacheEntry *tcache;		/* typcache entry for composite type */
+	uint64		tupdesc_id;		/* last-seen tupdesc identifier */
 } PLpgSQL_type;
 
-
 /*
- * PLpgSQL_datum is the common supertype for PLpgSQL_expr, PLpgSQL_var,
- * PLpgSQL_row, PLpgSQL_rec, PLpgSQL_recfield, and PLpgSQL_arrayelem
+ * SQL Query to plan and execute
  */
-typedef struct
-{								/* Generic datum array item		*/
-	int			dtype;
-	int			dno;
-} PLpgSQL_datum;
-
-/*
- * The variants PLpgSQL_var, PLpgSQL_row, and PLpgSQL_rec share these
- * fields
- */
-typedef struct
-{								/* Scalar or composite variable */
-	int			dtype;
-	int			dno;
-	char	   *refname;
-	int			lineno;
-} PLpgSQL_variable;
-
 typedef struct PLpgSQL_expr
-{								/* SQL Query to plan and execute	*/
-	int			dtype;
-	int			dno;
+{
 	char	   *query;
 	SPIPlanPtr	plan;
 	Bitmapset  *paramnos;		/* all dnos referenced by this query */
@@ -236,10 +231,10 @@ typedef struct PLpgSQL_expr
 	struct PLpgSQL_nsitem *ns;
 
 	/* fields for "simple expression" fast-path execution: */
-	Expr	   *expr_simple_expr;		/* NULL means not a simple expr */
+	Expr	   *expr_simple_expr;	/* NULL means not a simple expr */
 	int			expr_simple_generation; /* plancache generation we checked */
-	Oid			expr_simple_type;		/* result type Oid, if simple */
-	int32		expr_simple_typmod;		/* result typmod, if simple */
+	Oid			expr_simple_type;	/* result type Oid, if simple */
+	int32		expr_simple_typmod; /* result typmod, if simple */
 
 	/*
 	 * if expr is simple AND prepared in current transaction,
@@ -247,84 +242,188 @@ typedef struct PLpgSQL_expr
 	 * seeing if expr_simple_lxid matches current LXID.  (If not,
 	 * expr_simple_state probably points at garbage!)
 	 */
-	ExprState  *expr_simple_state;		/* eval tree for expr_simple_expr */
-	bool		expr_simple_in_use;		/* true if eval tree is active */
+	ExprState  *expr_simple_state;	/* eval tree for expr_simple_expr */
+	bool		expr_simple_in_use; /* true if eval tree is active */
 	LocalTransactionId expr_simple_lxid;
 } PLpgSQL_expr;
 
+/*
+ * Generic datum array item
+ *
+ * PLpgSQL_datum is the common supertype for PLpgSQL_var, PLpgSQL_row,
+ * PLpgSQL_rec, PLpgSQL_recfield, and PLpgSQL_arrayelem.
+ */
+typedef struct PLpgSQL_datum
+{
+	PLpgSQL_datum_type dtype;
+	int			dno;
+} PLpgSQL_datum;
 
-typedef struct
-{								/* Scalar variable */
-	int			dtype;
+/*
+ * Scalar or composite variable
+ *
+ * The variants PLpgSQL_var, PLpgSQL_row, and PLpgSQL_rec share these
+ * fields.
+ */
+typedef struct PLpgSQL_variable
+{
+	PLpgSQL_datum_type dtype;
 	int			dno;
 	char	   *refname;
 	int			lineno;
+	bool		isconst;
+	bool		notnull;
+	PLpgSQL_expr *default_val;
+} PLpgSQL_variable;
+
+/*
+ * Scalar variable
+ *
+ * DTYPE_VAR and DTYPE_PROMISE datums both use this struct type.
+ * A PROMISE datum works exactly like a VAR datum for most purposes,
+ * but if it is read without having previously been assigned to, then
+ * a special "promised" value is computed and assigned to the datum
+ * before the read is performed.  This technique avoids the overhead of
+ * computing the variable's value in cases where we expect that many
+ * functions will never read it.
+ */
+typedef struct PLpgSQL_var
+{
+	PLpgSQL_datum_type dtype;
+	int			dno;
+	char	   *refname;
+	int			lineno;
+	bool		isconst;
+	bool		notnull;
+	PLpgSQL_expr *default_val;
+	/* end of PLpgSQL_variable fields */
 
 	PLpgSQL_type *datatype;
-	int			isconst;
-	int			notnull;
-	PLpgSQL_expr *default_val;
+
+	/*
+	 * Variables declared as CURSOR FOR <query> are mostly like ordinary
+	 * scalar variables of type refcursor, but they have these additional
+	 * properties:
+	 */
 	PLpgSQL_expr *cursor_explicit_expr;
 	int			cursor_explicit_argrow;
 	int			cursor_options;
 
+	/* Fields below here can change at runtime */
+
 	Datum		value;
 	bool		isnull;
 	bool		freeval;
+
+	/*
+	 * The promise field records which "promised" value to assign if the
+	 * promise must be honored.  If it's a normal variable, or the promise has
+	 * been fulfilled, this is PLPGSQL_PROMISE_NONE.
+	 */
+	PLpgSQL_promise_type promise;
 } PLpgSQL_var;
 
-
-typedef struct
-{								/* Row variable */
-	int			dtype;
+/*
+ * Row variable - this represents one or more variables that are listed in an
+ * INTO clause, FOR-loop targetlist, cursor argument list, etc.  We also use
+ * a row to represent a function's OUT parameters when there's more than one.
+ *
+ * Note that there's no way to name the row as such from PL/pgSQL code,
+ * so many functions don't need to support these.
+ *
+ * That also means that there's no real name for the row variable, so we
+ * conventionally set refname to "(unnamed row)".  We could leave it NULL,
+ * but it's too convenient to be able to assume that refname is valid in
+ * all variants of PLpgSQL_variable.
+ *
+ * isconst, notnull, and default_val are unsupported (and hence
+ * always zero/null) for a row.  The member variables of a row should have
+ * been checked to be writable at compile time, so isconst is correctly set
+ * to false.  notnull and default_val aren't applicable.
+ */
+typedef struct PLpgSQL_row
+{
+	PLpgSQL_datum_type dtype;
 	int			dno;
 	char	   *refname;
 	int			lineno;
-
-	TupleDesc	rowtupdesc;
+	bool		isconst;
+	bool		notnull;
+	PLpgSQL_expr *default_val;
+	/* end of PLpgSQL_variable fields */
 
 	/*
-	 * Note: TupleDesc is only set up for named rowtypes, else it is NULL.
-	 *
-	 * Note: if the underlying rowtype contains a dropped column, the
-	 * corresponding fieldnames[] entry will be NULL, and there is no
-	 * corresponding var (varnos[] will be -1).
+	 * rowtupdesc is only set up if we might need to convert the row into a
+	 * composite datum, which currently only happens for OUT parameters.
+	 * Otherwise it is NULL.
 	 */
+	TupleDesc	rowtupdesc;
+
 	int			nfields;
 	char	  **fieldnames;
 	int		   *varnos;
 } PLpgSQL_row;
 
-
-typedef struct
-{								/* Record variable (non-fixed structure) */
-	int			dtype;
+/*
+ * Record variable (any composite type, including RECORD)
+ */
+typedef struct PLpgSQL_rec
+{
+	PLpgSQL_datum_type dtype;
 	int			dno;
 	char	   *refname;
 	int			lineno;
+	bool		isconst;
+	bool		notnull;
+	PLpgSQL_expr *default_val;
+	/* end of PLpgSQL_variable fields */
 
-	HeapTuple	tup;
-	TupleDesc	tupdesc;
-	bool		freetup;
-	bool		freetupdesc;
+	/*
+	 * Note: for non-RECORD cases, we may from time to time re-look-up the
+	 * composite type, using datatype->origtypname.  That can result in
+	 * changing rectypeid.
+	 */
+
+	PLpgSQL_type *datatype;		/* can be NULL, if rectypeid is RECORDOID */
+	Oid			rectypeid;		/* declared type of variable */
+	/* RECFIELDs for this record are chained together for easy access */
+	int			firstfield;		/* dno of first RECFIELD, or -1 if none */
+
+	/* Fields below here can change at runtime */
+
+	/* We always store record variables as "expanded" records */
+	ExpandedRecordHeader *erh;
 } PLpgSQL_rec;
 
-
-typedef struct
-{								/* Field in record */
-	int			dtype;
+/*
+ * Field in record
+ */
+typedef struct PLpgSQL_recfield
+{
+	PLpgSQL_datum_type dtype;
 	int			dno;
-	char	   *fieldname;
+	/* end of PLpgSQL_datum fields */
+
+	char	   *fieldname;		/* name of field */
 	int			recparentno;	/* dno of parent record */
+	int			nextfield;		/* dno of next child, or -1 if none */
+	uint64		rectupledescid; /* record's tupledesc ID as of last lookup */
+	ExpandedRecordFieldInfo finfo;	/* field's attnum and type info */
+	/* if rectupledescid == INVALID_TUPLEDESC_IDENTIFIER, finfo isn't valid */
 } PLpgSQL_recfield;
 
-
-typedef struct
-{								/* Element of array variable */
-	int			dtype;
+/*
+ * Element of array variable
+ */
+typedef struct PLpgSQL_arrayelem
+{
+	PLpgSQL_datum_type dtype;
 	int			dno;
+	/* end of PLpgSQL_datum fields */
+
 	PLpgSQL_expr *subscript;
 	int			arrayparentno;	/* dno of parent array variable */
+
 	/* Remaining fields are cached info about the array variable's type */
 	Oid			parenttypoid;	/* type of array variable; 0 if not yet set */
 	int32		parenttypmod;	/* typmod of array variable */
@@ -337,92 +436,162 @@ typedef struct
 	char		elemtypalign;	/* typalign of element type */
 } PLpgSQL_arrayelem;
 
-
+/*
+ * Item in the compilers namespace tree
+ */
 typedef struct PLpgSQL_nsitem
-{								/* Item in the compilers namespace tree */
-	int			itemtype;
+{
+	PLpgSQL_nsitem_type itemtype;
+
+	/*
+	 * For labels, itemno is a value of enum PLpgSQL_label_type. For other
+	 * itemtypes, itemno is the associated PLpgSQL_datum's dno.
+	 */
 	int			itemno;
-	/* For labels, itemno is a value of enum PLpgSQL_label_types. */
-	/* For other itemtypes, itemno is the associated PLpgSQL_datum's dno. */
 	struct PLpgSQL_nsitem *prev;
 	char		name[FLEXIBLE_ARRAY_MEMBER];	/* nul-terminated string */
 } PLpgSQL_nsitem;
 
-
-typedef struct
-{								/* Generic execution node		*/
-	int			cmd_type;
+/*
+ * Generic execution node
+ */
+typedef struct PLpgSQL_stmt
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 } PLpgSQL_stmt;
 
-
+/*
+ * One EXCEPTION condition name
+ */
 typedef struct PLpgSQL_condition
-{								/* One EXCEPTION condition name */
+{
 	int			sqlerrstate;	/* SQLSTATE code */
 	char	   *condname;		/* condition name (for debugging) */
 	struct PLpgSQL_condition *next;
 } PLpgSQL_condition;
 
-typedef struct
+/*
+ * EXCEPTION block
+ */
+typedef struct PLpgSQL_exception_block
 {
 	int			sqlstate_varno;
 	int			sqlerrm_varno;
 	List	   *exc_list;		/* List of WHEN clauses */
 } PLpgSQL_exception_block;
 
-typedef struct
-{								/* One EXCEPTION ... WHEN clause */
+/*
+ * One EXCEPTION ... WHEN clause
+ */
+typedef struct PLpgSQL_exception
+{
 	int			lineno;
 	PLpgSQL_condition *conditions;
 	List	   *action;			/* List of statements */
 } PLpgSQL_exception;
 
-
-typedef struct
-{								/* Block of statements			*/
-	int			cmd_type;
+/*
+ * Block of statements
+ */
+typedef struct PLpgSQL_stmt_block
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
 	List	   *body;			/* List of statements */
-	int			n_initvars;
-	int		   *initvarnos;
+	int			n_initvars;		/* Length of initvarnos[] */
+	int		   *initvarnos;		/* dnos of variables declared in this block */
 	PLpgSQL_exception_block *exceptions;
 } PLpgSQL_stmt_block;
 
-
-typedef struct
-{								/* Assign statement			*/
-	int			cmd_type;
+/*
+ * Assign statement
+ */
+typedef struct PLpgSQL_stmt_assign
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	int			varno;
 	PLpgSQL_expr *expr;
 } PLpgSQL_stmt_assign;
 
-typedef struct
-{								/* PERFORM statement		*/
-	int			cmd_type;
+/*
+ * PERFORM statement
+ */
+typedef struct PLpgSQL_stmt_perform
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *expr;
 } PLpgSQL_stmt_perform;
 
-typedef struct
-{								/* Get Diagnostics item		*/
-	int			kind;			/* id for diagnostic value desired */
+/*
+ * CALL statement
+ */
+typedef struct PLpgSQL_stmt_call
+{
+	PLpgSQL_stmt_type cmd_type;
+	int			lineno;
+	PLpgSQL_expr *expr;
+	bool		is_call;
+	PLpgSQL_variable *target;
+} PLpgSQL_stmt_call;
+
+/*
+ * COMMIT statement
+ */
+typedef struct PLpgSQL_stmt_commit
+{
+	PLpgSQL_stmt_type cmd_type;
+	int			lineno;
+} PLpgSQL_stmt_commit;
+
+/*
+ * ROLLBACK statement
+ */
+typedef struct PLpgSQL_stmt_rollback
+{
+	PLpgSQL_stmt_type cmd_type;
+	int			lineno;
+} PLpgSQL_stmt_rollback;
+
+/*
+ * SET statement
+ */
+typedef struct PLpgSQL_stmt_set
+{
+	PLpgSQL_stmt_type cmd_type;
+	int			lineno;
+	PLpgSQL_expr *expr;
+} PLpgSQL_stmt_set;
+
+/*
+ * GET DIAGNOSTICS item
+ */
+typedef struct PLpgSQL_diag_item
+{
+	PLpgSQL_getdiag_kind kind;	/* id for diagnostic value desired */
 	int			target;			/* where to assign it */
 } PLpgSQL_diag_item;
 
-typedef struct
-{								/* Get Diagnostics statement		*/
-	int			cmd_type;
+/*
+ * GET DIAGNOSTICS statement
+ */
+typedef struct PLpgSQL_stmt_getdiag
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	bool		is_stacked;		/* STACKED or CURRENT diagnostics area? */
 	List	   *diag_items;		/* List of PLpgSQL_diag_item */
 } PLpgSQL_stmt_getdiag;
 
-
-typedef struct
-{								/* IF statement				*/
-	int			cmd_type;
+/*
+ * IF statement
+ */
+typedef struct PLpgSQL_stmt_if
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *cond;			/* boolean expression for THEN */
 	List	   *then_body;		/* List of statements */
@@ -430,17 +599,22 @@ typedef struct
 	List	   *else_body;		/* List of statements */
 } PLpgSQL_stmt_if;
 
-typedef struct					/* one ELSIF arm of IF statement */
+/*
+ * one ELSIF arm of IF statement
+ */
+typedef struct PLpgSQL_if_elsif
 {
 	int			lineno;
 	PLpgSQL_expr *cond;			/* boolean expression for this case */
 	List	   *stmts;			/* List of statements */
 } PLpgSQL_if_elsif;
 
-
-typedef struct					/* CASE statement */
+/*
+ * CASE statement
+ */
+typedef struct PLpgSQL_stmt_case
 {
-	int			cmd_type;
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *t_expr;		/* test expression, or NULL if none */
 	int			t_varno;		/* var to store test expression value into */
@@ -449,36 +623,45 @@ typedef struct					/* CASE statement */
 	List	   *else_stmts;		/* List of statements */
 } PLpgSQL_stmt_case;
 
-typedef struct					/* one arm of CASE statement */
+/*
+ * one arm of CASE statement
+ */
+typedef struct PLpgSQL_case_when
 {
 	int			lineno;
 	PLpgSQL_expr *expr;			/* boolean expression for this case */
 	List	   *stmts;			/* List of statements */
 } PLpgSQL_case_when;
 
-
-typedef struct
-{								/* Unconditional LOOP statement		*/
-	int			cmd_type;
+/*
+ * Unconditional LOOP statement
+ */
+typedef struct PLpgSQL_stmt_loop
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
 	List	   *body;			/* List of statements */
 } PLpgSQL_stmt_loop;
 
-
-typedef struct
-{								/* WHILE cond LOOP statement		*/
-	int			cmd_type;
+/*
+ * WHILE cond LOOP statement
+ */
+typedef struct PLpgSQL_stmt_while
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
 	PLpgSQL_expr *cond;
 	List	   *body;			/* List of statements */
 } PLpgSQL_stmt_while;
 
-
-typedef struct
-{								/* FOR statement with integer loopvar	*/
-	int			cmd_type;
+/*
+ * FOR statement with integer loopvar
+ */
+typedef struct PLpgSQL_stmt_fori
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
 	PLpgSQL_var *var;
@@ -489,64 +672,70 @@ typedef struct
 	List	   *body;			/* List of statements */
 } PLpgSQL_stmt_fori;
 
-
 /*
  * PLpgSQL_stmt_forq represents a FOR statement running over a SQL query.
  * It is the common supertype of PLpgSQL_stmt_fors, PLpgSQL_stmt_forc
  * and PLpgSQL_dynfors.
  */
-typedef struct
+typedef struct PLpgSQL_stmt_forq
 {
-	int			cmd_type;
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
-	PLpgSQL_rec *rec;
-	PLpgSQL_row *row;
+	PLpgSQL_variable *var;		/* Loop variable (record or row) */
 	List	   *body;			/* List of statements */
 } PLpgSQL_stmt_forq;
 
-typedef struct
-{								/* FOR statement running over SELECT	*/
-	int			cmd_type;
+/*
+ * FOR statement running over SELECT
+ */
+typedef struct PLpgSQL_stmt_fors
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
-	PLpgSQL_rec *rec;
-	PLpgSQL_row *row;
+	PLpgSQL_variable *var;		/* Loop variable (record or row) */
 	List	   *body;			/* List of statements */
 	/* end of fields that must match PLpgSQL_stmt_forq */
 	PLpgSQL_expr *query;
 } PLpgSQL_stmt_fors;
 
-typedef struct
-{								/* FOR statement running over cursor	*/
-	int			cmd_type;
+/*
+ * FOR statement running over cursor
+ */
+typedef struct PLpgSQL_stmt_forc
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
-	PLpgSQL_rec *rec;
-	PLpgSQL_row *row;
+	PLpgSQL_variable *var;		/* Loop variable (record or row) */
 	List	   *body;			/* List of statements */
 	/* end of fields that must match PLpgSQL_stmt_forq */
 	int			curvar;
 	PLpgSQL_expr *argquery;		/* cursor arguments if any */
 } PLpgSQL_stmt_forc;
 
-typedef struct
-{								/* FOR statement running over EXECUTE	*/
-	int			cmd_type;
+/*
+ * FOR statement running over EXECUTE
+ */
+typedef struct PLpgSQL_stmt_dynfors
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
-	PLpgSQL_rec *rec;
-	PLpgSQL_row *row;
+	PLpgSQL_variable *var;		/* Loop variable (record or row) */
 	List	   *body;			/* List of statements */
 	/* end of fields that must match PLpgSQL_stmt_forq */
 	PLpgSQL_expr *query;
 	List	   *params;			/* USING expressions */
 } PLpgSQL_stmt_dynfors;
 
-
-typedef struct
-{								/* FOREACH item in array loop */
-	int			cmd_type;
+/*
+ * FOREACH item in array loop
+ */
+typedef struct PLpgSQL_stmt_foreach_a
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	char	   *label;
 	int			varno;			/* loop target variable */
@@ -555,27 +744,29 @@ typedef struct
 	List	   *body;			/* List of statements */
 } PLpgSQL_stmt_foreach_a;
 
-
-typedef struct
-{								/* OPEN a curvar					*/
-	int			cmd_type;
+/*
+ * OPEN a curvar
+ */
+typedef struct PLpgSQL_stmt_open
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	int			curvar;
 	int			cursor_options;
-	PLpgSQL_row *returntype;
 	PLpgSQL_expr *argquery;
 	PLpgSQL_expr *query;
 	PLpgSQL_expr *dynquery;
 	List	   *params;			/* USING expressions */
 } PLpgSQL_stmt_open;
 
-
-typedef struct
-{								/* FETCH or MOVE statement */
-	int			cmd_type;
+/*
+ * FETCH or MOVE statement
+ */
+typedef struct PLpgSQL_stmt_fetch
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
-	PLpgSQL_rec *rec;			/* target, as record or row */
-	PLpgSQL_row *row;
+	PLpgSQL_variable *target;	/* target (record or row) */
 	int			curvar;			/* cursor variable to fetch from */
 	FetchDirection direction;	/* fetch direction */
 	long		how_many;		/* count, if constant (expr is NULL) */
@@ -584,53 +775,68 @@ typedef struct
 	bool		returns_multiple_rows;	/* can return more than one row? */
 } PLpgSQL_stmt_fetch;
 
-
-typedef struct
-{								/* CLOSE curvar						*/
-	int			cmd_type;
+/*
+ * CLOSE curvar
+ */
+typedef struct PLpgSQL_stmt_close
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	int			curvar;
 } PLpgSQL_stmt_close;
 
-
-typedef struct
-{								/* EXIT or CONTINUE statement			*/
-	int			cmd_type;
+/*
+ * EXIT or CONTINUE statement
+ */
+typedef struct PLpgSQL_stmt_exit
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	bool		is_exit;		/* Is this an exit or a continue? */
 	char	   *label;			/* NULL if it's an unlabelled EXIT/CONTINUE */
 	PLpgSQL_expr *cond;
 } PLpgSQL_stmt_exit;
 
-
-typedef struct
-{								/* RETURN statement			*/
-	int			cmd_type;
+/*
+ * RETURN statement
+ */
+typedef struct PLpgSQL_stmt_return
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *expr;
 	int			retvarno;
 } PLpgSQL_stmt_return;
 
-typedef struct
-{								/* RETURN NEXT statement */
-	int			cmd_type;
+/*
+ * RETURN NEXT statement
+ */
+typedef struct PLpgSQL_stmt_return_next
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *expr;
 	int			retvarno;
 } PLpgSQL_stmt_return_next;
 
-typedef struct
-{								/* RETURN QUERY statement */
-	int			cmd_type;
+/*
+ * RETURN QUERY statement
+ */
+typedef struct PLpgSQL_stmt_return_query
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *query;		/* if static query */
 	PLpgSQL_expr *dynquery;		/* if dynamic query (RETURN QUERY EXECUTE) */
 	List	   *params;			/* USING arguments for dynamic query */
 } PLpgSQL_stmt_return_query;
 
-typedef struct
-{								/* RAISE statement			*/
-	int			cmd_type;
+/*
+ * RAISE statement
+ */
+typedef struct PLpgSQL_stmt_raise
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	int			elog_level;
 	char	   *condname;		/* condition name, SQLSTATE, or NULL */
@@ -639,49 +845,60 @@ typedef struct
 	List	   *options;		/* list of PLpgSQL_raise_option */
 } PLpgSQL_stmt_raise;
 
-typedef struct
-{								/* RAISE statement option */
-	int			opt_type;
+/*
+ * RAISE statement option
+ */
+typedef struct PLpgSQL_raise_option
+{
+	PLpgSQL_raise_option_type opt_type;
 	PLpgSQL_expr *expr;
 } PLpgSQL_raise_option;
 
-typedef struct
-{								/* ASSERT statement */
-	int			cmd_type;
+/*
+ * ASSERT statement
+ */
+typedef struct PLpgSQL_stmt_assert
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *cond;
 	PLpgSQL_expr *message;
 } PLpgSQL_stmt_assert;
 
-typedef struct
-{								/* Generic SQL statement to execute */
-	int			cmd_type;
+/*
+ * Generic SQL statement to execute
+ */
+typedef struct PLpgSQL_stmt_execsql
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *sqlstmt;
-	bool		mod_stmt;		/* is the stmt INSERT/UPDATE/DELETE? */
-	/* note: mod_stmt is set when we plan the query */
+	bool		mod_stmt;		/* is the stmt INSERT/UPDATE/DELETE?  Note:
+								 * mod_stmt is set when we plan the query */
 	bool		into;			/* INTO supplied? */
 	bool		strict;			/* INTO STRICT flag */
-	PLpgSQL_rec *rec;			/* INTO target, if record */
-	PLpgSQL_row *row;			/* INTO target, if row */
+	PLpgSQL_variable *target;	/* INTO target (record or row) */
 } PLpgSQL_stmt_execsql;
 
-
-typedef struct
-{								/* Dynamic SQL string to execute */
-	int			cmd_type;
+/*
+ * Dynamic SQL string to execute
+ */
+typedef struct PLpgSQL_stmt_dynexecute
+{
+	PLpgSQL_stmt_type cmd_type;
 	int			lineno;
 	PLpgSQL_expr *query;		/* string expression */
 	bool		into;			/* INTO supplied? */
 	bool		strict;			/* INTO STRICT flag */
-	PLpgSQL_rec *rec;			/* INTO target, if record */
-	PLpgSQL_row *row;			/* INTO target, if row */
+	PLpgSQL_variable *target;	/* INTO target (record or row) */
 	List	   *params;			/* USING expressions */
 } PLpgSQL_stmt_dynexecute;
 
-
+/*
+ * Hash lookup key for functions
+ */
 typedef struct PLpgSQL_func_hashkey
-{								/* Hash lookup key for functions */
+{
 	Oid			funcOid;
 
 	bool		isTrigger;		/* true if called as a DML trigger */
@@ -690,12 +907,12 @@ typedef struct PLpgSQL_func_hashkey
 	/* be careful that pad bytes in this struct get zeroed! */
 
 	/*
-	 * For a trigger function, the OID of the relation triggered on is part of
-	 * the hash key --- we want to compile the trigger separately for each
-	 * relation it is used with, in case the rowtype is different.  Zero if
-	 * not called as a DML trigger.
+	 * For a trigger function, the OID of the trigger is part of the hash key
+	 * --- we want to compile the trigger function separately for each trigger
+	 * it is used with, in case the rowtype or transition table names are
+	 * different.  Zero if not called as a DML trigger.
 	 */
-	Oid			trigrelOid;
+	Oid			trigOid;
 
 	/*
 	 * We must include the input collation as part of the hash key too,
@@ -711,6 +928,9 @@ typedef struct PLpgSQL_func_hashkey
 	Oid			argtypes[FUNC_MAX_ARGS];
 } PLpgSQL_func_hashkey;
 
+/*
+ * Trigger type
+ */
 typedef enum PLpgSQL_trigtype
 {
 	PLPGSQL_DML_TRIGGER,
@@ -718,8 +938,11 @@ typedef enum PLpgSQL_trigtype
 	PLPGSQL_NOT_TRIGGER
 } PLpgSQL_trigtype;
 
+/*
+ * Complete compiled function
+ */
 typedef struct PLpgSQL_function
-{								/* Complete compiled function	  */
+{
 	char	   *fn_signature;
 	Oid			fn_oid;
 	TransactionId fn_xmin;
@@ -733,8 +956,10 @@ typedef struct PLpgSQL_function
 	int			fn_rettyplen;
 	bool		fn_retbyval;
 	bool		fn_retistuple;
+	bool		fn_retisdomain;
 	bool		fn_retset;
 	bool		fn_readonly;
+	char		fn_prokind;
 
 	int			fn_nargs;
 	int			fn_argvarnos[FUNC_MAX_ARGS];
@@ -742,20 +967,6 @@ typedef struct PLpgSQL_function
 	int			found_varno;
 	int			new_varno;
 	int			old_varno;
-	int			tg_name_varno;
-	int			tg_when_varno;
-	int			tg_level_varno;
-	int			tg_op_varno;
-	int			tg_relid_varno;
-	int			tg_relname_varno;
-	int			tg_table_name_varno;
-	int			tg_table_schema_varno;
-	int			tg_nargs_varno;
-	int			tg_argv_varno;
-
-	/* for event triggers */
-	int			tg_event_varno;
-	int			tg_tag_varno;
 
 	PLpgSQL_resolve_option resolve_option;
 
@@ -768,7 +979,7 @@ typedef struct PLpgSQL_function
 	/* the datums representing the function's local variables */
 	int			ndatums;
 	PLpgSQL_datum **datums;
-	Bitmapset  *resettable_datums;		/* dnos of non-simple vars */
+	Size		copiable_size;	/* space for locally instantiated datums */
 
 	/* function body parsetree */
 	PLpgSQL_stmt_block *action;
@@ -778,11 +989,16 @@ typedef struct PLpgSQL_function
 	unsigned long use_count;
 } PLpgSQL_function;
 
-
+/*
+ * Runtime execution data
+ */
 typedef struct PLpgSQL_execstate
-{								/* Runtime execution data	*/
+{
 	PLpgSQL_function *func;		/* function being executed */
 
+	TriggerData *trigdata;		/* if regular trigger, data about firing */
+	EventTriggerData *evtrigdata;	/* if event trigger, data about firing */
+
 	Datum		retval;
 	bool		retisnull;
 	Oid			rettype;		/* type of current retval */
@@ -792,33 +1008,50 @@ typedef struct PLpgSQL_execstate
 	bool		retisset;
 
 	bool		readonly_func;
+	bool		atomic;
 
-	TupleDesc	rettupdesc;
 	char	   *exitlabel;		/* the "target" label of the current EXIT or
 								 * CONTINUE stmt, if any */
 	ErrorData  *cur_error;		/* current exception handler's error */
 
-	Tuplestorestate *tuple_store;		/* SRFs accumulate results here */
+	Tuplestorestate *tuple_store;	/* SRFs accumulate results here */
+	TupleDesc	tuple_store_desc;	/* descriptor for tuples in tuple_store */
 	MemoryContext tuple_store_cxt;
 	ResourceOwner tuple_store_owner;
 	ReturnSetInfo *rsi;
 
-	/* the datums representing the function's local variables */
 	int			found_varno;
+
+	/*
+	 * The datums representing the function's local variables.  Some of these
+	 * are local storage in this execstate, but some just point to the shared
+	 * copy belonging to the PLpgSQL_function, depending on whether or not we
+	 * need any per-execution state for the datum's dtype.
+	 */
 	int			ndatums;
 	PLpgSQL_datum **datums;
+	/* context containing variable values (same as func's SPI_proc context) */
+	MemoryContext datum_context;
 
-	/* we pass datums[i] to the executor, when needed, in paramLI->params[i] */
+	/*
+	 * paramLI is what we use to pass local variable values to the executor.
+	 * It does not have a ParamExternData array; we just dynamically
+	 * instantiate parameter data as needed.  By convention, PARAM_EXTERN
+	 * Params have paramid equal to the dno of the referenced local variable.
+	 */
 	ParamListInfo paramLI;
-	bool		params_dirty;	/* T if any resettable datum has been passed */
 
 	/* EState to use for "simple" expression evaluation */
 	EState	   *simple_eval_estate;
 
-	/* Lookup table to use for executing type casts */
+	/* lookup table to use for executing type casts */
 	HTAB	   *cast_hash;
 	MemoryContext cast_hash_context;
 
+	/* memory context for statement-lifespan temporary values */
+	MemoryContext stmt_mcontext;	/* current stmt context, or NULL if none */
+	MemoryContext stmt_mcontext_parent; /* parent of current context */
+
 	/* temporary state for results from evaluation of query or expr */
 	SPITupleTable *eval_tuptable;
 	uint64		eval_processed;
@@ -832,7 +1065,6 @@ typedef struct PLpgSQL_execstate
 	void	   *plugin_info;	/* reserved for use by optional plugin */
 } PLpgSQL_execstate;
 
-
 /*
  * A PLpgSQL_plugin structure represents an instrumentation plugin.
  * To instrument PL/pgSQL, a plugin library must access the rendezvous
@@ -847,24 +1079,23 @@ typedef struct PLpgSQL_execstate
  * (if the pointers are non-NULL) to give the plugin a chance to watch
  * what we are doing.
  *
- *	func_setup is called when we start a function, before we've initialized
- *	the local variables defined by the function.
+ * func_setup is called when we start a function, before we've initialized
+ * the local variables defined by the function.
  *
- *	func_beg is called when we start a function, after we've initialized
- *	the local variables.
+ * func_beg is called when we start a function, after we've initialized
+ * the local variables.
  *
- *	func_end is called at the end of a function.
+ * func_end is called at the end of a function.
  *
- *	stmt_beg and stmt_end are called before and after (respectively) each
- *	statement.
+ * stmt_beg and stmt_end are called before and after (respectively) each
+ * statement.
  *
  * Also, immediately before any call to func_setup, PL/pgSQL fills in the
  * error_callback and assign_expr fields with pointers to its own
  * plpgsql_exec_error_callback and exec_assign_expr functions.  This is
  * a somewhat ad-hoc expedient to simplify life for debugger plugins.
  */
-
-typedef struct
+typedef struct PLpgSQL_plugin
 {
 	/* Function pointers set up by the plugin */
 	void		(*func_setup) (PLpgSQL_execstate *estate, PLpgSQL_function *func);
@@ -876,24 +1107,25 @@ typedef struct
 	/* Function pointers set by PL/pgSQL itself */
 	void		(*error_callback) (void *arg);
 	void		(*assign_expr) (PLpgSQL_execstate *estate, PLpgSQL_datum *target,
-											PLpgSQL_expr *expr);
+								PLpgSQL_expr *expr);
 } PLpgSQL_plugin;
 
+/*
+ * Struct types used during parsing
+ */
 
-/* Struct types used during parsing */
-
-typedef struct
+typedef struct PLword
 {
 	char	   *ident;			/* palloc'd converted identifier */
 	bool		quoted;			/* Was it double-quoted? */
 } PLword;
 
-typedef struct
+typedef struct PLcword
 {
 	List	   *idents;			/* composite identifiers (list of String) */
 } PLcword;
 
-typedef struct
+typedef struct PLwdatum
 {
 	PLpgSQL_datum *datum;		/* referenced variable */
 	char	   *ident;			/* valid if simple name */
@@ -947,9 +1179,8 @@ extern PLpgSQL_plugin **plpgsql_plugin_ptr;
  * Function declarations
  **********************************************************************/
 
-/* ----------
+/*
  * Functions in pl_comp.c
- * ----------
  */
 extern PLpgSQL_function *plpgsql_compile(FunctionCallInfo fcinfo,
 				bool forValidator);
@@ -967,12 +1198,15 @@ extern PLpgSQL_type *plpgsql_parse_cwordtype(List *idents);
 extern PLpgSQL_type *plpgsql_parse_wordrowtype(char *ident);
 extern PLpgSQL_type *plpgsql_parse_cwordrowtype(List *idents);
 extern PLpgSQL_type *plpgsql_build_datatype(Oid typeOid, int32 typmod,
-					   Oid collation);
+					   Oid collation, TypeName *origtypname);
 extern PLpgSQL_variable *plpgsql_build_variable(const char *refname, int lineno,
 					   PLpgSQL_type *dtype,
 					   bool add2namespace);
 extern PLpgSQL_rec *plpgsql_build_record(const char *refname, int lineno,
+					 PLpgSQL_type *dtype, Oid rectypeid,
 					 bool add2namespace);
+extern PLpgSQL_recfield *plpgsql_build_recfield(PLpgSQL_rec *rec,
+					   const char *fldname);
 extern int plpgsql_recognize_err_condition(const char *condname,
 								bool allow_sqlstate);
 extern PLpgSQL_condition *plpgsql_parse_err_condition(char *condname);
@@ -980,19 +1214,18 @@ extern void plpgsql_adddatum(PLpgSQL_datum *newdatum);
 extern int	plpgsql_add_initdatums(int **varnos);
 extern void plpgsql_HashTableInit(void);
 
-/* ----------
+/*
  * Functions in pl_handler.c
- * ----------
  */
 extern void _PG_init(void);
 
-/* ----------
+/*
  * Functions in pl_exec.c
- * ----------
  */
 extern Datum plpgsql_exec_function(PLpgSQL_function *func,
 					  FunctionCallInfo fcinfo,
-					  EState *simple_eval_estate);
+					  EState *simple_eval_estate,
+					  bool atomic);
 extern HeapTuple plpgsql_exec_trigger(PLpgSQL_function *func,
 					 TriggerData *trigdata);
 extern void plpgsql_exec_event_trigger(PLpgSQL_function *func,
@@ -1006,16 +1239,15 @@ extern void plpgsql_exec_get_datum_type_info(PLpgSQL_execstate *estate,
 								 PLpgSQL_datum *datum,
 								 Oid *typeId, int32 *typMod, Oid *collation);
 
-/* ----------
+/*
  * Functions for namespace handling in pl_funcs.c
- * ----------
  */
 extern void plpgsql_ns_init(void);
 extern void plpgsql_ns_push(const char *label,
-				enum PLpgSQL_label_types label_type);
+				PLpgSQL_label_type label_type);
 extern void plpgsql_ns_pop(void);
 extern PLpgSQL_nsitem *plpgsql_ns_top(void);
-extern void plpgsql_ns_additem(int itemtype, int itemno, const char *name);
+extern void plpgsql_ns_additem(PLpgSQL_nsitem_type itemtype, int itemno, const char *name);
 extern PLpgSQL_nsitem *plpgsql_ns_lookup(PLpgSQL_nsitem *ns_cur, bool localmode,
 				  const char *name1, const char *name2,
 				  const char *name3, int *names_used);
@@ -1023,18 +1255,16 @@ extern PLpgSQL_nsitem *plpgsql_ns_lookup_label(PLpgSQL_nsitem *ns_cur,
 						const char *name);
 extern PLpgSQL_nsitem *plpgsql_ns_find_nearest_loop(PLpgSQL_nsitem *ns_cur);
 
-/* ----------
+/*
  * Other functions in pl_funcs.c
- * ----------
  */
 extern const char *plpgsql_stmt_typename(PLpgSQL_stmt *stmt);
-extern const char *plpgsql_getdiag_kindname(int kind);
+extern const char *plpgsql_getdiag_kindname(PLpgSQL_getdiag_kind kind);
 extern void plpgsql_free_function_memory(PLpgSQL_function *func);
 extern void plpgsql_dumptree(PLpgSQL_function *func);
 
-/* ----------
+/*
  * Scanner functions in pl_scanner.c
- * ----------
  */
 extern int	plpgsql_base_yylex(void);
 extern int	plpgsql_yylex(void);
@@ -1052,10 +1282,9 @@ extern int	plpgsql_latest_lineno(void);
 extern void plpgsql_scanner_init(const char *str);
 extern void plpgsql_scanner_finish(void);
 
-/* ----------
+/*
  * Externs in gram.y
- * ----------
  */
 extern int	plpgsql_yyparse(void);
 
-#endif   /* PLPGSQL_H */
+#endif							/* PLPGSQL_H */
